/* -*- mode: C; c-basic-offset: 4; indent-tabs-mode: nil -*- */
#include "pygpgme.h"

struct pygpgme_constant {
  const char name[28];
  long value;
};

#define CONST(name) { #name, GPGME_##name }

static const struct pygpgme_constant constants[] = {
  /* gpgme_data_encoding_t */
  CONST(DATA_ENCODING_NONE),
  CONST(DATA_ENCODING_BINARY),
  CONST(DATA_ENCODING_BASE64),
  CONST(DATA_ENCODING_ARMOR),

  /* gpgme_pubkey_algo_t */
  CONST(PK_RSA),
  CONST(PK_RSA_E),
  CONST(PK_RSA_S),
  CONST(PK_ELG_E),
  CONST(PK_DSA),
  CONST(PK_ELG),

  /* gpgme_hash_algo_t */
  CONST(MD_NONE),
  CONST(MD_MD5),
  CONST(MD_SHA1),
  CONST(MD_RMD160),
  CONST(MD_MD2),
  CONST(MD_TIGER),
  CONST(MD_HAVAL),
  CONST(MD_SHA256),
  CONST(MD_SHA384),
  CONST(MD_SHA512),
  CONST(MD_MD4),
  CONST(MD_CRC32),
  CONST(MD_CRC32_RFC1510),
  CONST(MD_CRC24_RFC2440),

  /* gpgme_sig_mode_t */
  CONST(SIG_MODE_NORMAL),
  CONST(SIG_MODE_DETACH),
  CONST(SIG_MODE_CLEAR),

  /* gpgme_validity_t */
  CONST(VALIDITY_UNKNOWN),
  CONST(VALIDITY_UNDEFINED),
  CONST(VALIDITY_NEVER),
  CONST(VALIDITY_MARGINAL),
  CONST(VALIDITY_FULL),
  CONST(VALIDITY_ULTIMATE),

  /* gpgme_protocol_t */
  CONST(PROTOCOL_OpenPGP),
  CONST(PROTOCOL_CMS),

  /* gpgme_keylist_mode_t */
  CONST(KEYLIST_MODE_LOCAL),
  CONST(KEYLIST_MODE_EXTERN),
  CONST(KEYLIST_MODE_SIGS),
  CONST(KEYLIST_MODE_VALIDATE),

  /* gpgme_status_code_t */
  CONST(STATUS_EOF),
  CONST(STATUS_ENTER),
  CONST(STATUS_LEAVE),
  CONST(STATUS_ABORT),
  CONST(STATUS_GOODSIG),
  CONST(STATUS_BADSIG),
  CONST(STATUS_ERRSIG),
  CONST(STATUS_BADARMOR),
  CONST(STATUS_RSA_OR_IDEA),
  CONST(STATUS_KEYEXPIRED),
  CONST(STATUS_KEYREVOKED),
  CONST(STATUS_TRUST_UNDEFINED),
  CONST(STATUS_TRUST_NEVER),
  CONST(STATUS_TRUST_MARGINAL),
  CONST(STATUS_TRUST_FULLY),
  CONST(STATUS_TRUST_ULTIMATE),
  CONST(STATUS_SHM_INFO),
  CONST(STATUS_SHM_GET),
  CONST(STATUS_SHM_GET_BOOL),
  CONST(STATUS_SHM_GET_HIDDEN),
  CONST(STATUS_NEED_PASSPHRASE),
  CONST(STATUS_VALIDSIG),
  CONST(STATUS_SIG_ID),
  CONST(STATUS_ENC_TO),
  CONST(STATUS_NODATA),
  CONST(STATUS_BAD_PASSPHRASE),
  CONST(STATUS_NO_PUBKEY),
  CONST(STATUS_NO_SECKEY),
  CONST(STATUS_NEED_PASSPHRASE_SYM),
  CONST(STATUS_DECRYPTION_FAILED),
  CONST(STATUS_DECRYPTION_OKAY),
  CONST(STATUS_MISSING_PASSPHRASE),
  CONST(STATUS_GOOD_PASSPHRASE),
  CONST(STATUS_GOODMDC),
  CONST(STATUS_BADMDC),
  CONST(STATUS_ERRMDC),
  CONST(STATUS_IMPORTED),
  CONST(STATUS_IMPORT_OK),
  CONST(STATUS_IMPORT_PROBLEM),
  CONST(STATUS_IMPORT_RES),
  CONST(STATUS_FILE_START),
  CONST(STATUS_FILE_DONE),
  CONST(STATUS_FILE_ERROR),
  CONST(STATUS_BEGIN_DECRYPTION),
  CONST(STATUS_END_DECRYPTION),
  CONST(STATUS_BEGIN_ENCRYPTION),
  CONST(STATUS_END_ENCRYPTION),
  CONST(STATUS_DELETE_PROBLEM),
  CONST(STATUS_GET_BOOL),
  CONST(STATUS_GET_LINE),
  CONST(STATUS_GET_HIDDEN),
  CONST(STATUS_GOT_IT),
  CONST(STATUS_PROGRESS),
  CONST(STATUS_SIG_CREATED),
  CONST(STATUS_SESSION_KEY),
  CONST(STATUS_NOTATION_NAME),
  CONST(STATUS_NOTATION_DATA),
  CONST(STATUS_POLICY_URL),
  CONST(STATUS_BEGIN_STREAM),
  CONST(STATUS_END_STREAM),
  CONST(STATUS_KEY_CREATED),
  CONST(STATUS_USERID_HINT),
  CONST(STATUS_UNEXPECTED),
  CONST(STATUS_INV_RECP),
  CONST(STATUS_NO_RECP),
  CONST(STATUS_ALREADY_SIGNED),
  CONST(STATUS_SIGEXPIRED),
  CONST(STATUS_EXPSIG),
  CONST(STATUS_EXPKEYSIG),
  CONST(STATUS_TRUNCATED),
  CONST(STATUS_ERROR),
  CONST(STATUS_NEWSIG),
  CONST(STATUS_REVKEYSIG),

  /* gpgme_encrypt_flags_t */
  CONST(ENCRYPT_ALWAYS_TRUST),

  /* gpgme_sigsum_t */
  CONST(SIGSUM_VALID),
  CONST(SIGSUM_GREEN),
  CONST(SIGSUM_RED),
  CONST(SIGSUM_KEY_REVOKED),
  CONST(SIGSUM_KEY_EXPIRED),
  CONST(SIGSUM_SIG_EXPIRED),
  CONST(SIGSUM_KEY_MISSING),
  CONST(SIGSUM_CRL_MISSING),
  CONST(SIGSUM_CRL_TOO_OLD),
  CONST(SIGSUM_BAD_POLICY),
  CONST(SIGSUM_SYS_ERROR),

  /* import status */
  CONST(IMPORT_NEW),
  CONST(IMPORT_UID),
  CONST(IMPORT_SIG),
  CONST(IMPORT_SUBKEY),
  CONST(IMPORT_SECRET),
};

static const int n_constants = sizeof(constants) / sizeof(constants[0]);

PyObject *
pygpgme_make_constants(PyObject *self, PyObject *args)
{
    PyObject *dict;
    int i;

    if (!PyArg_ParseTuple(args, "O!", &PyDict_Type, &dict))
        return NULL;

    for (i = 0; i < n_constants; i++) {
        PyObject *item;

        item = PyInt_FromLong(constants[i].value);
        PyDict_SetItemString(dict, constants[i].name, item);
        Py_DECREF(item);
    }
    Py_RETURN_NONE;
}
